@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2008, 2014 Sonatype Inc. and others.
+ * Copyright (c) 2008, 2017 Sonatype Inc. and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -9,6 +9,7 @@
  *    Sonatype Inc. - initial API and implementation
  *    SAP SE - port to surefire 2.10
  *    Mickael Istria (Red Hat Inc.) - 386988 Support for provisioned applications
+ *    Bachmann electrontic GmbH - 510425 parallel mode requires threadCount>1 or useUnlimitedThreads=true
  ******************************************************************************/
 package org.eclipse.tycho.surefire;
 
@@ -20,6 +21,7 @@ import java.net.MalformedURLException;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
+import java.util.HashMap;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
@@ -44,7 +46,10 @@ import org.apache.maven.plugins.annotations.Parameter;
 import org.apache.maven.plugins.annotations.ResolutionScope;
 import org.apache.maven.project.MavenProject;
 import org.apache.maven.repository.RepositorySystem;
+import org.apache.maven.surefire.booter.BooterConstants;
+import org.apache.maven.surefire.booter.PropertiesWrapper;
 import org.apache.maven.surefire.booter.ProviderParameterNames;
+import org.apache.maven.surefire.testset.TestListResolver;
 import org.apache.maven.surefire.util.DefaultScanResult;
 import org.apache.maven.surefire.util.ScanResult;
 import org.apache.maven.toolchain.Toolchain;
@@ -115,7 +120,6 @@ public class TestMojo extends AbstractMojo {
      */
     @Parameter(defaultValue = "${project.build.directory}/work")
     private File work;
-
     /**
      * <a href=
      * "http://help.eclipse.org/juno/topic/org.eclipse.platform.doc.isv/reference/misc/runtime-options.html#osgiinstancearea"
@@ -144,7 +148,7 @@ public class TestMojo extends AbstractMojo {
 
     /**
      * List of patterns (separated by commas) used to specify the tests that should be included in
-     * testing. When not specified and whent the <code>test</code> parameter is not specified, the
+     * testing. When not specified and when the <code>test</code> parameter is not specified, the
      * default includes will be
      * <code>**&#47;Test*.java   **&#47;*Test.java   **&#47;*TestCase.java</code>
      */
@@ -406,6 +410,23 @@ public class TestMojo extends AbstractMojo {
     @Parameter
     private BundleStartLevel defaultStartLevel;
 
+    /**
+     * Flaky tests will re-run until they pass or the number of reruns has been exhausted. See
+     * surefire documentation for details.
+     * <p>
+     * Note: This feature is supported only for JUnit 4.x
+     * </p>
+     */
+    @Parameter(property = "surefire.rerunFailingTestsCount", defaultValue = "0")
+    private Integer rerunFailingTestsCount;
+
+    /**
+     * Skips the remaining tests after the Nth failure or error. See surefire documentation for
+     * details.
+     */
+    @Parameter(property = "surefire.skipAfterFailureCount", defaultValue = "0")
+    private Integer skipAfterFailureCount;
+
     @Component
     private RepositorySystem repositorySystem;
 
@@ -453,6 +474,14 @@ public class TestMojo extends AbstractMojo {
     @Parameter(defaultValue = "filesystem")
     private String runOrder;
 
+    /**
+     * When {@code true}, stack traces are trimmed to only show lines within the test.
+     *
+     * @since 1.3.0
+     */
+    @Parameter(property = "trimStackTrace", defaultValue = "true")
+    private boolean trimStackTrace;
+
     /**
      * (JUnit 4.7 provider) Supports values "classes"/"methods"/"both" to run in separate threads,
      * as controlled by threadCount.
@@ -592,8 +621,8 @@ public class TestMojo extends AbstractMojo {
      * <li>BREE: use MANIFEST header <code>Bundle-RequiredExecutionEnvironment</code> to lookup the
      * JDK from
      * <a href="http://maven.apache.org/guides/mini/guide-using-toolchains.html">toolchains.xml</a>.
-     * The value of BREE will be matched against the id of the toolchain elements in toolchains.xml.
-     * </li>
+     * The value of BREE will be matched against the id of the toolchain elements in
+     * toolchains.xml.</li>
      * </ul>
      * 
      * Example for BREE: <br>
@@ -622,6 +651,23 @@ public class TestMojo extends AbstractMojo {
     @Parameter(defaultValue = "SYSTEM")
     private JDKUsage useJDK;
 
+    /**
+     * Only supported by the TestNG test provider. The values specified are passed to TestNG as test
+     * suite files. The suite files will overwrite the {@link #includes} and {@link #excludes}
+     * patterns. The path to the suite file(s) could be relative (test bundle classpath) or an
+     * absolute path to xml files outside the test bundle.
+     * 
+     * <pre>
+     * &lt;configuration&gt;
+     *   &lt;suiteXmlFiles&gt;
+     *     &lt;suiteXmlFile&gt;myTestSuite.xml&lt;/suiteXmlFile&gt;
+     *   &lt;/suiteXmlFiles&gt;
+     * &lt;/configuration&gt;
+     * </pre>
+     */
+    @Parameter
+    private List<String> suiteXmlFiles;
+
     @Override
     public void execute() throws MojoExecutionException, MojoFailureException {
         if (shouldSkip()) {
@@ -845,30 +891,46 @@ public class TestMojo extends AbstractMojo {
     }
 
     private void createSurefireProperties(TestFrameworkProvider provider) throws MojoExecutionException {
-        Properties p = new Properties();
-        p.put("testpluginname", getTestBundleSymbolicName());
-        p.put("testclassesdirectory", testClassesDirectory.getAbsolutePath());
-        p.put("reportsdirectory", reportsDirectory.getAbsolutePath());
-        p.put("redirectTestOutputToFile", String.valueOf(redirectTestOutputToFile));
+        PropertiesWrapper wrapper = new PropertiesWrapper(new HashMap<String, String>());
+        wrapper.setProperty("testpluginname", getTestBundleSymbolicName());
+        wrapper.setProperty("testclassesdirectory", testClassesDirectory.getAbsolutePath());
+        wrapper.setProperty("reportsdirectory", reportsDirectory.getAbsolutePath());
+        wrapper.setProperty("redirectTestOutputToFile", String.valueOf(redirectTestOutputToFile));
 
-        p.put("failifnotests", String.valueOf(failIfNoTests));
-        p.put("runOrder", runOrder);
+        wrapper.setProperty("failifnotests", String.valueOf(failIfNoTests));
+        wrapper.setProperty("runOrder", runOrder);
+        wrapper.setProperty("trimStackTrace", String.valueOf(trimStackTrace));
+        wrapper.setProperty("skipAfterFailureCount", String.valueOf(skipAfterFailureCount));
+        wrapper.setProperty("rerunFailingTestsCount", String.valueOf(rerunFailingTestsCount));
         Properties mergedProviderProperties = getMergedProviderProperties();
+        mergedProviderProperties.putAll(provider.getProviderSpecificProperties());
         ScanResult scanResult = scanForTests();
-        scanResult.writeTo(mergedProviderProperties);
-        for (Map.Entry<?, ?> entry : mergedProviderProperties.entrySet()) {
-            p.put("__provider." + entry.getKey(), entry.getValue());
+        Map<String, String> providerPropertiesAsMap = propertiesAsMap(mergedProviderProperties);
+        scanResult.writeTo(providerPropertiesAsMap);
+        for (Map.Entry<String, String> entry : providerPropertiesAsMap.entrySet()) {
+            wrapper.setProperty("__provider." + entry.getKey(), entry.getValue().toString());
         }
-        p.setProperty("testprovider", provider.getSurefireProviderClassName());
+        wrapper.setProperty("testprovider", provider.getSurefireProviderClassName());
         getLog().debug("Using test framework provider " + provider.getClass().getName());
-        storeProperties(p, surefireProperties);
+        wrapper.addList(suiteXmlFiles, BooterConstants.TEST_SUITE_XML_FILES);
+        storeProperties(wrapper.getProperties(), surefireProperties);
     }
 
-    private Properties getMergedProviderProperties() {
+    protected Properties getMergedProviderProperties() throws MojoExecutionException {
         Properties result = new Properties();
         result.putAll(providerProperties);
         if (parallel != null) {
             result.put(ProviderParameterNames.PARALLEL_PROP, parallel.name());
+            if (!useUnlimitedThreads) {
+                if (perCoreThreadCount && threadCount < 1) {
+                    throw new MojoExecutionException(
+                            "Parallel mode with perCoreThreadCount=true requires threadCount>=1");
+                }
+                if (!perCoreThreadCount && threadCount <= 1) {
+                    throw new MojoExecutionException(
+                            "Parallel mode requires threadCount>1 or useUnlimitedThreads=true");
+                }
+            }
             if (threadCount > 0) {
                 result.put(ProviderParameterNames.THREADCOUNT_PROP, String.valueOf(threadCount));
             }
@@ -911,13 +973,18 @@ public class TestMojo extends AbstractMojo {
         } else {
             excludeList = defaultExcludes;
         }
-        DirectoryScanner scanner = new DirectoryScanner(testClassesDirectory, includeList, excludeList,
-                Collections.<String> emptyList());
+        // TODO bug 495353 we should we rather let TestListResolver do the work here 
+        // by passing in the unparsed String or Strings instead of already parsed include/exclude list
+        // (this would add support for running single test methods, negation etc.)
+        TestListResolver resolver = new TestListResolver(includeList, excludeList);
+        DirectoryScanner scanner = new DirectoryScanner(testClassesDirectory, resolver);
         DefaultScanResult scanResult = scanner.scan();
         return scanResult;
     }
 
-    private void storeProperties(Properties p, File file) throws MojoExecutionException {
+    private void storeProperties(Map<String, String> propertiesMap, File file) throws MojoExecutionException {
+        Properties p = new Properties();
+        p.putAll(propertiesMap);
         try {
             BufferedOutputStream out = new BufferedOutputStream(new FileOutputStream(file));
             try {
@@ -980,7 +1047,8 @@ public class TestMojo extends AbstractMojo {
 
         default:
             throw new MojoFailureException("An unexpected error occured while launching the test runtime (return code "
-                    + result + "). See log for details.");
+                    + result + "). See log " + new File(osgiDataDirectory, ".metadata/.log").getAbsolutePath()
+                    + " for details.");
         }
     }
 
@@ -1071,11 +1139,19 @@ public class TestMojo extends AbstractMojo {
         if (eeConfig.isCustomProfile()) {
             Properties customProfileProps = eeConfig.getFullSpecification().getProfileProperties();
             File profileFile = new File(new File(project.getBuild().getDirectory()), "custom.profile");
-            storeProperties(customProfileProps, profileFile);
+            storeProperties(propertiesAsMap(customProfileProps), profileFile);
             cli.addVMArguments("-D" + EquinoxConfiguration.PROP_OSGI_JAVA_PROFILE + "=" + profileFile.toURI());
         }
     }
 
+    private Map<String, String> propertiesAsMap(Properties p) {
+        Map<String, String> result = new HashMap<String, String>();
+        for (String entry : p.stringPropertyNames()) {
+            result.put(entry, p.getProperty(entry));
+        }
+        return result;
+    }
+
     void addProgramArgs(EquinoxLaunchConfiguration cli, String... arguments) {
         if (arguments != null) {
             for (String argument : arguments) {
